;******************************
;* Program Test Magnetic Card *
;* Read Track2 Modulo5 Format *
;* HardWare    : ET CP2051 V2 *
;* Display Data Code to RS232 *
;* CPU Control : MCS51        *
;* Complier    : SXA51        *
;******************************
;
DATA_MAG    EQU     P1.0                ; Data From Magnetic Card
PRESENT     EQU     P1.1                ; Enable Magnetic Card
CLK_MAG     EQU     P1.2                ; Clock Sync Magnetic Card

            ORG     20H
FLAG_BUF:   DS      1                   ; Flag Buffer
CHECKSUM:   DS      1
BUFFER:     DS      40                  ; 40 Byte Buffer
;
START       EQU     FLAG_BUF.0          ; Start Flag Status
STOP        EQU     FLAG_BUF.1          ; Stop Flag Status
ERROR       EQU     FLAG_BUF.2          ; Error Flag Status
PARITY      EQU     FLAG_BUF.3          ; Parity Error Flag Status

            ORG     0000H
PROGRAM:    MOV     SP,#128-32          ; Stack 32 Byte
            MOV     FLAG_BUF,#0         ; Clear All Flag
            SETB    DATA_MAG            ; Standy Signale
            SETB    PRESENT
            SETB    CLK_MAG
            ;
INIT_SR:    MOV     A,#0FDH             ; Set baud rate 9600
            MOV     TH1,A
            MOV     TL1,A
            MOV     TMOD,#00100000B     ; TIMER1 MODE 1
            CLR     ES                  ; Disable serial interupt
            CLR     ET1                 ; Disable timer1 interupt
            SETB    TR1                 ; Set timer1 control
            MOV     SCON,#01010000B     ; Serial mode 1
            ;
TEST_MAG:   LCALL   PRINT_SER
            DB      'TEST MAGNETIC CARD',0DH,0AH,00
            ;
DSP_NUM0:   LCALL   GET_DATA            ; Get Number of Card
            MOV     R0,#BUFFER
            JB      ERROR,SUM_ERR
            JB      PARITY,PAR_ERR
            SJMP    DSP_NUM1
            ;
SUM_ERR:    LCALL   PRINT_SER           ; Checksum Error
            DB      'Checksum Error!',0DH,0AH,00
            CLR     ERROR
            SJMP    DSP_NUM0
            ;
PAR_ERR:    LCALL   PRINT_SER           ; Parity Error
            DB      'Parity Error!',0DH,0AH,00
            CLR     PARITY
            SJMP    DSP_NUM0
            ;
DSP_NUM1:   MOV     A,@R0
            CJNE    A,#0FH,DSP_NUM2
            ADD     A,#30H
            LCALL   TX_BYTE             ; Display End
            SJMP    DSP_NUM3
DSP_NUM2:   ADD     A,#30H
            LCALL   TX_BYTE
            INC     R0
            SJMP    DSP_NUM1
DSP_NUM3:   MOV     A,#0DH              ; Carrier Return
            LCALL   TX_BYTE
            MOV     A,#0AH              ; Line Feed
            LCALL   TX_BYTE
            SJMP    DSP_NUM0

;*********************
;* Clear Data Buffer *
;* Register : R0,ACC *
;*********************
;
CLR_BUFF:   PUSH    B
            MOV     B,#40
            MOV     R0,#BUFFER          ; Clear Data Buffer
CLR_BUF1:   CLR     A
            MOV     @R0,A
            INC     R0
            DJNZ    B,CLR_BUF1
            POP     B
            RET

;********************************
;* Read Data From Magnetic Card *
;* Get Number of Card (Modulo5) *
;* Output : BUFFER (BCD 40Byte) *
;* Reg.   : R1,R2,ACC,CHECKSUM  *
;********************************
;
GET_DATA:   LCALL   CLR_BUFF            ; Clear Buffer
            MOV     CHECKSUM,#0
            MOV     R1,#BUFFER-1        ; Pointer to Save Data
            CLR     START               ; Clear Any Flag
            CLR     STOP
            CLR     ERROR
            ;
GET_DAT0:   LCALL   GET_CLK             ; Get Start Sentinel
            MOV     C,DATA_MAG
            CPL     C
            JB      DATA_MAG,GET_DAT0   ; Loop Until Start Bit
            SETB    START               ; Strat Bit OK
            ;
GET_DAT1:   MOV     R2,#5               ; Modulo 5 Format Counter
            CLR     A
            CLR     PARITY
            INC     R1                  ; Point to Next Byte Save
GET_DAT2:   JB      START,GET_DAT3      ; Get 1-Byte Data
            LCALL   GET_CLK
            MOV     C,DATA_MAG
            CPL     C
GET_DAT3:   RRC     A
            CLR     START
            DJNZ    R2,GET_DAT2
            RR      A
            RR      A
            RR      A
            JB      P,GET_DAT4          ; Parity Odd OK
            SETB    PARITY              ; Parity Error
            SJMP    GET_DAT8
            ;
GET_DAT4:   ANL     A,#0FH              ; Ignore Parity
            JB      STOP,GET_DAT5
            PUSH    ACC
            XRL     A,CHECKSUM          ; Checksum Data
            MOV     CHECKSUM,A
            POP     ACC
GET_DAT5:   MOV     @R1,A               ; Save Data
            JB      STOP,GET_DAT6       ; Stop Operation
            CJNE    A,#0FH,GET_DAT1
            SETB    STOP
            SJMP    GET_DAT1
GET_DAT6:   CJNE    A,CHECKSUM,GET_DAT7
            CLR     ERROR               ; Checksum OK
            SJMP    GET_DAT8
GET_DAT7:   SETB    ERROR               ; Checksum Error
GET_DAT8:   RET

;**********************
;* Get Magnetic Clock *
;**********************
;
GET_CLK:    JB      CLK_MAG,$           ; Wait Falling Clock
            NOP
            NOP
            JNB     CLK_MAG,$           ; Wait Rising Clock
            NOP
            NOP
            RET

;*************************
;* Send 1-Byte to RS-232 *
;* Input   : ACC         *
;* Output  : Serial port *
;*************************
;
TX_BYTE:    CLR     TI
            MOV     SBUF,A
            JNB     TI,$
            CLR     TI
            RET

;************************************
;*  Print Data to Serial Port RS232 *
;*  Usage    : LCALL PRINT_SER      *
;*           : DB   'xxxx',00       *
;*  Register : ACC                  *
;*  Note     : last byte must be 00 *
;************************************
;
PRINT_SER:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   TX_BYTE
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

            END

