;******************************
;* Program Test Magnetic Card *
;* Read Track2 Modulo5 Format *
;* HardWare    : ET CP2051 V2 *
;* Display By Dot. LCD 32 x 2 *
;* CPU Control : MCS51        *
;* Complier    : SXA51        *
;******************************
;
CS_LCD      EQU     P3.3                ; E  LCD (PIN INT1)
RS_LCD      EQU     P3.4                ; RS LCD (Pin T0)
;
DATA_MAG    EQU     P1.0                ; Data From Magnetic Card
PRESENT     EQU     P1.1                ; Enable Magnetic Card
CLK_MAG     EQU     P1.2                ; Clock Sync Magnetic Card

            ORG     20H
FLAG_BUF:   DS      1                   ; Flag Buffer
CHECKSUM:   DS      1
BUFFER:     DS      40                  ; 40 Byte Buffer
;
START       EQU     FLAG_BUF.0          ; Start Flag Status
STOP        EQU     FLAG_BUF.1          ; Stop Flag Status
ERROR       EQU     FLAG_BUF.2          ; Error Flag Status
PARITY      EQU     FLAG_BUF.3          ; Parity Error Flag Status

            ORG     0000H
PROGRAM:    MOV     SP,#128-32          ; Stack 32 Byte
            MOV     FLAG_BUF,#0         ; Clear All Flag
            SETB    DATA_MAG            ; Standy Signale
            SETB    PRESENT
            SETB    CLK_MAG
            LCALL   INIT_LCD            ; Initial LCD 4Bit
            ;
TEST_MAG:   MOV     A,#00H              ; LCD Line1
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      ' DEMO FOR  MAGNETIC CARD READER ',00
            ;
DSP_NUM0:   LCALL   GET_DATA            ; Get Number of Card
            MOV     A,#40H
            LCALL   GOTO_LCD            ; LCD Line2
            LCALL   PRINT_LCD
            DB      '                                ',00
            MOV     A,#40H
            LCALL   GOTO_LCD            ; LCD Line2
            MOV     R0,#BUFFER
            JB      ERROR,SUM_ERR
            JB      PARITY,PAR_ERR
            AJMP    DSP_NUM1
            ;
SUM_ERR:    LCALL   PRINT_LCD           ; Checksum Error
            DB      'Checksum Error!',00
            CLR     ERROR
            AJMP    DSP_NUM0
            ;
PAR_ERR:    LCALL   PRINT_LCD           ; Parity Error
            DB      'Parity Error!',00
            CLR     PARITY
            AJMP    DSP_NUM0
            ;
DSP_NUM1:   MOV     A,@R0
            CJNE    A,#0FH,DSP_NUM2
            ADD     A,#30H
            LCALL   WR_LCD              ; Display End
            AJMP    DSP_NUM0
DSP_NUM2:   ADD     A,#30H
            LCALL   WR_LCD
            INC     R0
            AJMP    DSP_NUM1

;*********************
;* Clear Data Buffer *
;* Register : R0,ACC *
;*********************
;
CLR_BUFF:   PUSH    B
            MOV     B,#40
            MOV     R0,#BUFFER          ; Clear Data Buffer
CLR_BUF1:   CLR     A
            MOV     @R0,A
            INC     R0
            DJNZ    B,CLR_BUF1
            POP     B
            RET

;********************************
;* Read Data From Magnetic Card *
;* Get Number of Card (Modulo5) *
;* Output : BUFFER (BCD 40Byte) *
;* Reg.   : R1,R2,ACC,CHECKSUM  *
;********************************
;
GET_DATA:   SETB    DATA_MAG            ; Standy Signale
            SETB    PRESENT
            SETB    CLK_MAG
            LCALL   CLR_BUFF            ; Clear Buffer
            MOV     CHECKSUM,#0
            MOV     R1,#BUFFER-1        ; Pointer to Save Data
            CLR     START               ; Clear Any Flag
            CLR     STOP
            CLR     ERROR
            ;
GET_DAT0:   LCALL   GET_CLK             ; Get Start Sentinel
            MOV     C,DATA_MAG
            CPL     C
            JB      DATA_MAG,GET_DAT0   ; Loop Until Start Bit
            SETB    START               ; Strat Bit OK
            ;
GET_DAT1:   MOV     R2,#5               ; Modulo 5 Format Counter
            CLR     A
            CLR     PARITY
            INC     R1                  ; Point to Next Byte Save
GET_DAT2:   JB      START,GET_DAT3      ; Get 1-Byte Data
            LCALL   GET_CLK
            MOV     C,DATA_MAG
            CPL     C
GET_DAT3:   RRC     A
            CLR     START
            DJNZ    R2,GET_DAT2
            RR      A
            RR      A
            RR      A
            JB      P,GET_DAT4          ; Parity Odd OK
            SETB    PARITY              ; Parity Error
            SJMP    GET_DAT8
            ;
GET_DAT4:   ANL     A,#0FH              ; Ignore Parity
            JB      STOP,GET_DAT5
            PUSH    ACC
            XRL     A,CHECKSUM          ; Checksum Data
            MOV     CHECKSUM,A
            POP     ACC
GET_DAT5:   MOV     @R1,A               ; Save Data
            JB      STOP,GET_DAT6       ; Stop Operation
            CJNE    A,#0FH,GET_DAT1
            SETB    STOP
            SJMP    GET_DAT1
GET_DAT6:   CJNE    A,CHECKSUM,GET_DAT7
            CLR     ERROR               ; Checksum OK
            SJMP    GET_DAT8
GET_DAT7:   SETB    ERROR               ; Checksum Error
GET_DAT8:   RET

;**********************
;* Get Magnetic Clock *
;**********************
;
GET_CLK:    JB      CLK_MAG,$           ; Wait Falling Clock
            NOP
            NOP
            JNB     CLK_MAG,$           ; Wait Rising Clock
            NOP
            NOP
            RET

;************************************
;*  Print Data Box to DotMatrix LCD *
;*  Usage    : LCALL PRINT_LCD      *
;*           : DB   'xxxx',00       *
;*  Register : ACC                  *
;*  Note     : last byte must be 00 *
;************************************
;
PRINT_LCD:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            AJMP    PRINT3
PRINT2:     LCALL   WR_LCD
            INC     DPTR
            AJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

;**************************
;*   Write ASCII to LCD   *
;* Input  : ACC  (ASCII)  *
;* Output : Data bus LCD  *
;**************************
;
WR_LCD:     PUSH    B
            SETB    RS_LCD          ; Write Data select
            MOV     B,A
            ANL     A,#0F0H
            MOV     P1,A            ; High byte
            LCALL   EN_LCD
            MOV     A,B             ; Low byte
            SWAP    A
            ANL     A,#0F0H
            MOV     P1,A
            LCALL   EN_LCD
            POP     B
            RET

;**************************
;* Write Instruction LCD  *
;* Input  : ACC (Command) *
;* Output : Data bus LCD  *
;**************************
;
WR_INS:     PUSH    B
            CLR     RS_LCD          ; Instruction select
            MOV     B,A
            ANL     A,#0F0H
            MOV     P1,A            ; High byte
            LCALL   EN_LCD
            MOV     A,B             ; Low byte
            SWAP    A
            ANL     A,#0F0H
            MOV     P1,A
            LCALL   EN_LCD
            POP     B
            RET

;************************
;* Goto position of LCD *
;* Input : ACC (addr.)  *
;************************
;
GOTO_LCD:   SETB    ACC.7
            LCALL   WR_INS
            RET

;*******************
;*   Initial LCD   *
;* 4-Bit Interface *
;*******************
;
INIT_LCD:   CLR     RS_LCD
            MOV     A,#33H          ; Set DL=1 3-time
            LCALL   WR_INS
            MOV     A,#32H          ; Clear DL=0 1-time
            LCALL   WR_INS
            MOV     A,#28H          ; Function set
            LCALL   WR_INS          ; DL=0 4Bit,N=1 2Line,F=0 5X7
            MOV     A,#0CH          ; Display on/off Control
            LCALL   WR_INS          ; Entry display,cursor off,cursor not blink
            MOV     A,#06H          ; Entry mode set
            LCALL   WR_INS          ; I/D=1 Increment,S=0 cursor shift
            MOV     A,#01H          ; Clear display
            LCALL   WR_INS          ; clear display,set DD RAM address=0
            RET

;**********************
;*  Enable Pin E LCD  *
;* Active Chip select *
;**********************
;
EN_LCD:     CLR     CS_LCD          ; Enable LCD
            LCALL   BUSY            ; Busy delay time
            SETB    CS_LCD          ; Disable LCD
            RET

;***********************
;* Delay time for Busy *
;*    Wait LCD Ready   *
;***********************
;
BUSY:       MOV     R7,#0FFH
            DJNZ    R7,$
            RET

            END





