;******************************
;* Program Test Magnetic Card *
;* Read Track2 Modulo5 Format *
;* Display By  Dot matrix LCD *
;* HardWare    : ET CP-SB31   *
;* CPU Control : MCS51        *
;* Complier    : SXA51        *
;******************************
;
WR_INSLCD   EQU     0E0C0H              ; Instruction Port of LCD
RD_INSLCD   EQU     0E0C1H              ; Read Busy Port LCD
WR_DATLCD   EQU     0E0C2H              ; Write Character LCD
RD_DATLCD   EQU     0E0C3H              ; Read DDRAM From LCD
;
DATA_MAG    EQU     P1.0                ; Data From Magnetic Card
PRESENT     EQU     P1.1                ; Enable Magnetic Card
CLK_MAG     EQU     P1.2                ; Clock Sync Magnetic Card
;
FLAG_BUF    EQU     20H                 ; Flag Buffer
CHECKSUM    EQU     21H
START       EQU     FLAG_BUF.0          ; Start Flag Status
STOP        EQU     FLAG_BUF.1          ; Stop Flag Status
ERROR       EQU     FLAG_BUF.2          ; Error Flag Status
PARITY      EQU     FLAG_BUF.3          ; Parity Error Flag Status

            ORG     2200H
PROGRAM:    MOV     SP,#128-32          ; Stack 32 Byte
            MOV     FLAG_BUF,#0         ; Clear All Flag
            SETB    DATA_MAG            ; Standy Signale
            SETB    PRESENT
            SETB    CLK_MAG
            LCALL   INIT_LCD            ; Initial LCD 8Bit
            ;
TEST_MAG:   MOV     A,#00H              ; LCD Line1
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      ' DEMO FOR  MAGNETIC CARD READER ',00
            ;
DSP_NUM0:   LCALL   GET_DATA            ; Get Number of Card
            MOV     DPTR,#BUFFER
            JB      ERROR,SUM_ERR
            JB      PARITY,PAR_ERR
            ;
            PUSH    DPH
            PUSH    DPL
            MOV     A,#40H              ; LCD Line2
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD           ; Checksum Error
            DB      '                                ',00
            MOV     A,#40H              ; LCD Line2
            LCALL   GOTO_LCD
            ;
            POP     DPL
            POP     DPH
            SJMP    DSP_NUM1
            ;
SUM_ERR:    MOV     A,#40H              ; LCD Line2
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD           ; Checksum Error
            DB      'Checksum Error!                 ',00
            CLR     ERROR
            LJMP    DSP_NUM0
            ;
PAR_ERR:    MOV     A,#40H              ; LCD Line2
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD           ; Parity Error
            DB      'Parity Error!                   ',00
            CLR     PARITY
            LJMP    DSP_NUM0
            ;
DSP_NUM1:   MOVX    A,@DPTR
            CJNE    A,#0FH,DSP_NUM2
            ADD     A,#30H
            LCALL   WR_LCD              ; Display End
            SJMP    DSP_NUM3
DSP_NUM2:   ADD     A,#30H
            LCALL   WR_LCD
            INC     DPTR
            SJMP    DSP_NUM1
DSP_NUM3:   LJMP    DSP_NUM0

;*********************
;* Clear Data Buffer *
;*********************
;
CLR_BUFF:   PUSH    B
            MOV     B,#00H
            MOV     DPTR,#BUFFER        ; Clear Data Buffer
CLR_BUF1:   CLR     A
            MOVX    @DPTR,A
            INC     DPTR
            DJNZ    B,CLR_BUF1
            POP     B
            RET

;********************************
;* Read Data From Magnetic Card *
;* Get Number of Card (Modulo5) *
;********************************
;
GET_DATA:   LCALL   CLR_BUFF            ; Clear Buffer
            MOV     CHECKSUM,#0
            MOV     DPTR,#BUFFER-1      ; Pointer to Save Data
            CLR     START               ; Clear Any Flag
            CLR     STOP
            CLR     ERROR
            ;
GET_DAT0:   LCALL   GET_CLK             ; Get Start Sentinel
            MOV     C,DATA_MAG
            CPL     C
            JB      DATA_MAG,GET_DAT0   ; Loop Until Start Bit
            SETB    START               ; Strat Bit OK
            ;
GET_DAT1:   MOV     B,#5                ; Modulo 5 Format Counter
            CLR     A
            CLR     PARITY
            INC     DPTR                ; Point to Next Byte Save
GET_DAT2:   JB      START,GET_DAT3      ; Get 1-Byte Data
            LCALL   GET_CLK
            MOV     C,DATA_MAG
            CPL     C
GET_DAT3:   RRC     A
            CLR     START
            DJNZ    B,GET_DAT2
            RR      A
            RR      A
            RR      A
            JB      P,GET_DAT4          ; Parity Odd OK
            SETB    PARITY              ; Parity Error
            SJMP    GET_DAT8
            ;
GET_DAT4:   ANL     A,#0FH              ; Ignore Parity
            JB      STOP,GET_DAT5
            PUSH    ACC
            XRL     A,CHECKSUM          ; Checksum Data
            MOV     CHECKSUM,A
            POP     ACC
GET_DAT5:   MOVX    @DPTR,A             ; Save Data
            JB      STOP,GET_DAT6       ; Stop Operation
            CJNE    A,#0FH,GET_DAT1
            SETB    STOP
            SJMP    GET_DAT1
GET_DAT6:   CJNE    A,CHECKSUM,GET_DAT7
            CLR     ERROR               ; Checksum OK
            SJMP    GET_DAT8
GET_DAT7:   SETB    ERROR               ; Checksum Error
GET_DAT8:   RET


;**********************
;* Get Magnetic Clock *
;**********************
;
GET_CLK:    JB      CLK_MAG,$           ; Wait Falling Clock
            JNB     CLK_MAG,$           ; Wait Rising Clock
            NOP
            NOP
            NOP
            NOP
            RET

;************************************
;*  Print Data data to Serial Port  *
;*  to display buffer.              *
;*  Usage    : LCALL PRINT_LCD      *
;*           : DB   'xxxx',00       *
;*  Register : ACC                  *
;*  Note     : last byte must be 00 *
;************************************
;
PRINT_LCD:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   WR_LCD
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

;***********************
;* Initial LCD Display *
;* 8-Bit Interface LCD *
;***********************
;
INIT_LCD:   MOV     DPTR,#WR_INSLCD         ; Write Instruction
            MOV     A,#00111000B            ; Function Set
            MOVX    @DPTR,A                 ; DL=1 8bit,N=1 1/16 duty,F=0 5x7
            LCALL   BUSY_LCD                ; Wait LCD Busy
            ;
            MOV     A,#00001100B            ; Display on/off Control
            MOVX    @DPTR,A                 ; D=1 off,C=0 cursor off,B=0 not blink
            LCALL   BUSY_LCD                ; Wait LCD Busy
            ;
            MOV     A,#00000110B            ; Entry Mode Set
            MOVX    @DPTR,A                 ; I/D=1 increment,S=0 right
            LCALL   BUSY_LCD                ; Wait LCD Busy
            ;
            MOV     A,#00000001B            ; Clear Display LCD all digit
            MOVX    @DPTR,A
            LCALL   BUSY_LCD                ; Wait LCD Busy
            RET

;************************
;*  Read Busy Flag LCD  *
;************************
;
BUSY_LCD:   PUSH    DPL
            PUSH    DPH
            PUSH    ACC
            MOV     DPTR,#RD_INSLCD
BUSY1:      MOVX    A,@DPTR
            JB      ACC.7,BUSY1
            POP     ACC
            POP     DPH
            POP     DPL
            RET

;************************
;* GOTO Position of LCD *
;* Input : A (Address)  *
;************************
;
GOTO_LCD:   PUSH    DPL
            PUSH    DPH
            LCALL   BUSY_LCD                ; Wait LCD Busy
            MOV     DPTR,#WR_INSLCD
            SETB    ACC.7                   ; Set DD RAM
            MOVX    @DPTR,A
            POP     DPH
            POP     DPL
            RET

;************************
;*  Wrire ASCII to LCD  *
;*  Input :  A (ASCII)  *
;************************
;
WR_LCD:     PUSH    DPL
            PUSH    DPH
            LCALL   BUSY_LCD                ; Wait LCD Busy
            MOV     DPTR,#WR_DATLCD
            MOVX    @DPTR,A
            POP     DPH
            POP     DPL
            RET

BUFFER:     DS      40

            END


;*************************
;* Wrire Data LCD Line 1 *
;* Show Receive Com Port *
;* Input :  A (ASCII)    *
;*************************
;
WR_LCD:     MOV     A,#00H                  ; Goto LCD Line1
            LCALL   GOTO_LCD
            MOV     R0,#LCD_SBUF
            MOV     R2,#20
WR_SLCD1:   MOV     A,@R0
            LCALL   WR_LCD
            INC     R0
            DJNZ    R2,WR_SLCD1
            RET

;*************************
;* Wrire Data LCD Line 2 *
;* Show Enter Keyboard   *
;* Input :  A (ASCII)    *
;*************************
;
WR_KLCD:    LCALL   SHF_LBUF                ; MSB <= LSB <= ASCII
WR_KLCD1:   MOV     A,#40H                  ; Goto LCD Line2
            LCALL   GOTO_LCD
            MOV     R0,#LCD_KBUF
            MOV     R2,#20
WR_KLCD2:   MOV     A,@R0
            LCALL   WR_LCD
            INC     R0
            DJNZ    R2,WR_KLCD2
            RET



