;******************************
;* Program Test Magnetic Card *
;* Read Track2 Modulo5 Format *
;* HardWare    : ET CP-SB31   *
;* CPU Control : MCS51        *
;* Complier    : SXA51        *
;******************************
;
DATA_MAG    EQU     P1.0                ; Data From Magnetic Card
PRESENT     EQU     P1.1                ; Enable Magnetic Card
CLK_MAG     EQU     P1.2                ; Clock Sync Magnetic Card
;
FLAG_BUF    EQU     20H                 ; Flag Buffer
CHECKSUM    EQU     21H
START       EQU     FLAG_BUF.0          ; Start Flag Status
STOP        EQU     FLAG_BUF.1          ; Stop Flag Status
ERROR       EQU     FLAG_BUF.2          ; Error Flag Status
PARITY      EQU     FLAG_BUF.3          ; Parity Error Flag Status

            ORG     2200H
PROGRAM:    MOV     SP,#128-32          ; Stack 32 Byte
            MOV     FLAG_BUF,#0         ; Clear All Flag
            SETB    DATA_MAG            ; Standy Signale
            SETB    PRESENT
            SETB    CLK_MAG
            ;
            MOV     P1,#0FFH
INIT_SR:    MOV     A,#0FDH             ; Set baud rate 9600
            MOV     TH1,A
            MOV     TL1,A
            MOV     TMOD,#00100000B     ; TIMER1 MODE 1
            CLR     ES                  ; Disable serial interupt
            CLR     ET1                 ; Disable timer1 interupt
            SETB    TR1                 ; Set timer1 control
            MOV     SCON,#01010000B     ; Serial mode 1
            ;
TEST_MAG:   LCALL   PRINT_SER
            DB      'TEST MAGNETIC CARD',0DH,0AH,00
            ;
DSP_NUM0:   LCALL   GET_DATA            ; Get Number of Card
            MOV     DPTR,#BUFFER
            JB      ERROR,SUM_ERR
            JB      PARITY,PAR_ERR
            SJMP    DSP_NUM1
            ;
SUM_ERR:    LCALL   PRINT_SER           ; Checksum Error
            DB      'Checksum Error!',0DH,0AH,00
            CLR     ERROR
            SJMP    DSP_NUM0
            ;
PAR_ERR:    LCALL   PRINT_SER           ; Parity Error
            DB      'Parity Error!',0DH,0AH,00
            CLR     PARITY
            SJMP    DSP_NUM0
            ;
DSP_NUM1:   MOVX    A,@DPTR
            CJNE    A,#0FH,DSP_NUM2
            ADD     A,#30H
            LCALL   TX_BYTE             ; Display End
            SJMP    DSP_NUM3
DSP_NUM2:   ADD     A,#30H
            LCALL   TX_BYTE
            INC     DPTR
            SJMP    DSP_NUM1
DSP_NUM3:   MOV     A,#0DH              ; Carrier Return
            LCALL   TX_BYTE
            MOV     A,#0AH              ; Line Feed
            LCALL   TX_BYTE
            SJMP    DSP_NUM0

;*********************
;* Clear Data Buffer *
;*********************
;
CLR_BUFF:   PUSH    B
            MOV     B,#00H
            MOV     DPTR,#BUFFER        ; Clear Data Buffer
CLR_BUF1:   CLR     A
            MOVX    @DPTR,A
            INC     DPTR
            DJNZ    B,CLR_BUF1
            POP     B
            RET

;********************************
;* Read Data From Magnetic Card *
;* Get Number of Card (Modulo5) *
;********************************
;
GET_DATA:   LCALL   CLR_BUFF            ; Clear Buffer
            MOV     CHECKSUM,#0
            MOV     DPTR,#BUFFER-1      ; Pointer to Save Data
            CLR     START               ; Clear Any Flag
            CLR     STOP
            CLR     ERROR
            ;
GET_DAT0:   LCALL   GET_CLK             ; Get Start Sentinel
            MOV     C,DATA_MAG
            CPL     C
            JB      DATA_MAG,GET_DAT0   ; Loop Until Start Bit
            SETB    START               ; Strat Bit OK
            ;
GET_DAT1:   MOV     B,#5                ; Modulo 5 Format Counter
            CLR     A
            CLR     PARITY
            INC     DPTR                ; Point to Next Byte Save
GET_DAT2:   JB      START,GET_DAT3      ; Get 1-Byte Data
            LCALL   GET_CLK
            MOV     C,DATA_MAG
            CPL     C
GET_DAT3:   RRC     A
            CLR     START
            DJNZ    B,GET_DAT2
            RR      A
            RR      A
            RR      A
            JB      P,GET_DAT4          ; Parity Odd OK
            SETB    PARITY              ; Parity Error
            SJMP    GET_DAT8
            ;
GET_DAT4:   ANL     A,#0FH              ; Ignore Parity
            JB      STOP,GET_DAT5
            PUSH    ACC
            XRL     A,CHECKSUM          ; Checksum Data
            MOV     CHECKSUM,A
            POP     ACC
GET_DAT5:   MOVX    @DPTR,A             ; Save Data
            JB      STOP,GET_DAT6       ; Stop Operation
            CJNE    A,#0FH,GET_DAT1
            SETB    STOP
            SJMP    GET_DAT1
GET_DAT6:   CJNE    A,CHECKSUM,GET_DAT7
            CLR     ERROR               ; Checksum OK
            SJMP    GET_DAT8
GET_DAT7:   SETB    ERROR               ; Checksum Error
GET_DAT8:   RET


;**********************
;* Get Magnetic Clock *
;**********************
;
GET_CLK:    JB      CLK_MAG,$           ; Wait Falling Clock
            JNB     CLK_MAG,$           ; Wait Rising Clock
            NOP
            NOP
            NOP
            NOP
            RET

;*************************
;* Send 1-Byte to RS-232 *
;* Input   : ACC         *
;* Output  : Serial port *
;*************************
;
TX_BYTE:    CLR     TI
            MOV     SBUF,A
            JNB     TI,$
            CLR     TI
            RET

;************************************
;*  Print Data data to Serial Port  *
;*  to display buffer.              *
;*  Usage    : LCALL PRINT_SER      *
;*           : DB   'xxxx',00       *
;*  Register : ACC                  *
;*  Note     : last byte must be 00 *
;************************************
;
PRINT_SER:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   TX_BYTE
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

BUFFER:     DS      40

            END

;******************************
;*    Send 2 byte of ASCII    *
;*  Input : ACC (HEX)         *
;*  Example : A = A3H         *
;*          : send 41H,33H    *
;******************************
;
TX_2ASC:    LCALL   HEX_2ASC
            ;LCALL   TX_BYTE
            MOV     A,B
            LCALL   TX_BYTE
            RET

;******************************
;*   Convert HEX to ASCII     *
;* Input    :  A              *
;* Output   :  A (high-byte)  *
;*          :  B (low-byte)   *
;******************************
;
HEX_2ASC:   PUSH    ACC
            LCALL   HEX_TO_ASC
            MOV     B,A                 ; B = Lo byte
            POP     ACC
            SWAP    A
            LCALL   HEX_TO_ASC
            RET

;********************************
;*  Convert hex (0-F) to ASCII  *
;*  Input   : ACC (HEX code)    *
;*  Output  : ACC (ASCII code)  *
;********************************
;
HEX_TO_ASC: PUSH    PSW
            CLR     CY
            ANL     A,#00001111B
            CJNE    A,#09H,HTOA1
            SETB    CY
HTOA1:      JNB     CY,HTOA2            ;> 0AH
            ORL     A,#30H              ;< 0AH
            SJMP    END_HTOA
HTOA2:      SUBB    A,#09H              ;> 0AH
            ORL     A,#40H
END_HTOA:   POP     PSW
            RET
